//--------------------------------------
// Procedures sous-programme arpegiateur
//--------------------------------------


//----------------------------------------------------
//----------------------------------------------------
//------------------  ARPEGIATEUR --------------------
//----------------------------------------------------
//----------------------------------------------------

//-------------------------------
// Initialisation de larpegiateur
//-------------------------------
void InitArp(void)
{
  for (i=0; i<MAXNOTEARP;i++)   // Initialisation du tableau des note actives
      NoteArp[i]=99;            // 255=note non jouee sinon valeur pitch
  CountNote=0;                  // RAZ compteur de note
  LastCountNote=99;             // Compteur de note precedent
  NbNoteArp=0;                  // Note dans arpegiateur
  LastNbNoteArp=0;              // Derniere note jouee
  PosSeq=-1;                     // Position courante dans la sequence
  InitSequenceArp();            // Reinitialiser la sequence a zero
}

void InitSequenceArp(void)
{
   memset(SequenceArp,0,sizeof(SequenceArp));
}

//-------------------------------------------
// Tri du tableau des notes de larpegiateur
// pour les egrener dans l'ordre du clavier
//-------------------------------------------
void TriNoteArp(void) {
byte temp,swap; 
  // Boucle infinie pour le tri
  while(1) {
    swap=0;
    for (i=0; i<MAXNOTEARP-1; i++) {
       if (NoteArp[i]>NoteArp[i+1]) { // Si 2 elements consecutif dans le desordre
          temp=NoteArp[i];            // swap des valeurs
          NoteArp[i]=NoteArp[i+1];
          NoteArp[i+1]=temp;
          swap=1;
       }
    }
    if (swap==0)                      // Si aucun swap          
       return;                        // On sort le tableau est trie
  }
}


//---------------------------------------------	
// HANDLER CONTINUE arpegiateur
// Gestion du message continue pour arpegiateur
//---------------------------------------------	
void MyHandleContinueArp(void) {
  ArpStart=1;
}


//------------------------------------------
// HANDLER START arpegiateur	
// Gestion du message start pour arpegiateur
//------------------------------------------	
void MyHandleStartArp(void) {
  ArpStart=1;
  ClockMidi=0;
  PosSeq=-1;
}

//-----------------------------------------
// HANDLER STOP arpegiateur
// Gestion du message stop pour arpegiateur
//-----------------------------------------
void MyHandleStopArp(void) {
  InitArp();
  ClockMidi=0;
  PosSeq=-1;
  // Si une note midi encore en cours de jeu
  // on l'arrete
  if (LastNoteMidiArp) {
  		MIDI.sendNoteOff(LastNoteMidiArp,0,1);
		LastNoteMidiArp=0;
  }
}

//----------------------------------------------------- 
// HANDLER CLOCK arpegiateur
// Procedure appellee a chaque tick de l'horloge maitre
// 1 tick= 1/24eme de noire
//----------------------------------------------------- 
void  MyHandleClockArp(void) {
  ClockMidi++;
  if (ClockMidi==1) {       // Si debut de division
      if (PosSeq==-1)
          PosSeq=0;
      PlayNoteArp();      // On joue
  }    
  
  //else
  //  digitalWrite(GATE1,LOW);
      
  if (ClockMidi==24/ListeClockMenu[ClockDivider]) {
      ClockMidi=0;     
      PosSeq++;
      
      //digitalWrite(GATE1,LOW);
      //delay(5);
      if (SequenceArp[PosSeq]==0) {
          PosSeq=0;
      }
  }
}

//---------------------------------
// Joue une note de la sequence arp
// Avec la conversion de midi -> CV
//---------------------------------
void PlayNoteArp(void)
{                                      
  if (NbNoteArp==0)      // Si aucune note dans arpegiateur
    return;            // On sort
  vdac=0;              
  // test du type de CV
  switch(AModeCV) {
      // Volt par octave
      case VOCT :
          if (SequenceArp[PosSeq]>=36 and SequenceArp[PosSeq] <=96) {
              vdac = pgm_read_word_near( VOct + SequenceArp[PosSeq] - 36);
          }
          else {
              PosSeq=0;
              if (SequenceArp[PosSeq]>=36 and SequenceArp[PosSeq] <=96) {
                  vdac = pgm_read_word_near( VOct + SequenceArp[PosSeq] - 36);
              } 
          } 
          break;
      // Hertz par volt
      case HVOLT :
          if (SequenceArp[PosSeq]>=36 and SequenceArp[PosSeq] <=87) {
              vdac = pgm_read_word_near( HVolt + SequenceArp[PosSeq] - 36);
          } 
         
          break;
      // SX150
      case SX150 :
          break;
  };
  
  // ecriture de la valeur en tension dans le dac
  writeDAC(DAC1,vdac);
  // Ouverture du GATE
  digitalWrite(GATE1,HIGH);
  StartGate=millis();
  if (LastNoteMidiArp) {
  		MIDI.sendNoteOff(LastNoteMidiArp,0,1);
		LastNoteMidiArp=0;
  }
  MIDI.sendNoteOn(SequenceArp[PosSeq],127,1);
  LastNoteMidiArp=SequenceArp[PosSeq];
}

//-----------------------------------------------
// Fabrique la sequence complete arpegee
// En fonction de letendue des octaves (de 1 a 3)
// et du type de motif
//-----------------------------------------------
void ReBuildSequenceArp(void)
{
byte IndexSeq;
byte i,j,k;
  IndexSeq=0;
  PosSeq=-1;                             // remettre a zero position dans sequence
  InitSequenceArp();                    // Effacer la sequence complete
  switch(NumMotifArp) {
    case MOTIF_UP:
      for (i=0; i<OctArp; i++ ) {     // Pour chaque octave
        for (j=0; j<NbNoteArp; j++) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j]+(i*12);
            IndexSeq++; 
        }
      }
      break;    
    case MOTIF_DOWN: 
      // note inverse et octave down
      // en partant de n octave au dessus
      //-------------------------------------     
      for (i=OctArp; i>0; i-- ) {     // Pour chaque octave
        for (j=NbNoteArp; j>0; j--) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j-1]+((i-1)*12);
            IndexSeq++; 
        }
      }
        break;
            
    case MOTIF_UPDOWN:
      // Motif UP d'abord
      for (i=0; i<OctArp; i++ ) {     // Pour chaque octave
        for (j=0; j<NbNoteArp; j++) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j]+(i*12);
            IndexSeq++; 
        }
      }
      // Motif down en partant n octave au dessus
      // et on redescend les notes a l'envers
      for (i=OctArp; i>0; i-- ) {     // Pour chaque octave
        for (j=NbNoteArp; j>0; j--) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j-1]+((i-1)*12);
            IndexSeq++; 
        }
      }
        
      break;    
    case MOTIF_DOWNUP:    
      // Motif down en partant n octave au dessus
      // et on redescend les notes a l'envers
      for (i=OctArp; i>0; i-- ) {     // Pour chaque octave
        for (j=NbNoteArp; j>0; j--) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j-1]+((i-1)*12);
            IndexSeq++; 
        }
      }
      // Motif UP ensuite
      for (i=0; i<OctArp; i++ ) {     // Pour chaque octave
        for (j=0; j<NbNoteArp; j++) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j]+(i*12);
            IndexSeq++; 
        }
      }
      break;    
    case MOTIF_OCTAVEUP:  
        for (j=0; j<NbNoteArp; j++) {  // Pour chaque note appuyee
            for (i=0; i<OctArp; i++ ) {     // Pour chaque octave
                SequenceArp[IndexSeq]=NoteArp[j]+(i*12);
                IndexSeq++; 
            }
      }

      break;    
    case MOTIF_OCTAVEDOWN:
        
        for (j=NbNoteArp; j>0; j--) {  // Pour chaque note appuyee
            for (i=OctArp; i>0; i-- ) {     // Pour chaque octave
                SequenceArp[IndexSeq]=NoteArp[j-1]-((i-1)*12);
                IndexSeq++; 
            }
      }
      

      break;    
    case MOTIF_PENDULE:
      // Creation du motif de base   
      for (i=0; i<OctArp; i++ ) {     // Pour chaque octave
        for (j=0; j<NbNoteArp; j++) {  // Pour chaque note appuyee
          SequenceArp[IndexSeq]=NoteArp[j]+(i*12);
          IndexSeq++; 
        }
      }
      
      // Transformation en mode pendulum
      memcpy(SequenceArp+24,SequenceArp,24);  // Transfert du motif en haut
      k=NbNoteArp*OctArp; // Nombre total de note a transferer
      j=0;
      for (i=0;i<k; i++) {
        SequenceArp[i]=SequenceArp[j+24];
        j++;
        i++;
        if (i<k) {
            SequenceArp[i]=SequenceArp[24+k-j];
        }
      }
      memset(SequenceArp+24,0, 24);
         
      break;    
    case MOTIF_RANDOM:
    
      for (i=0; i<OctArp; i++ ) {     // Pour chaque octave
        for (j=0; j<NbNoteArp; j++) {  // Pour chaque note appuyee
            SequenceArp[IndexSeq]=NoteArp[j]+(i*12);
            IndexSeq++; 
        }
      }
      memcpy(SequenceArp+24,SequenceArp,24);  // Transfert du motif en haut
      k=NbNoteArp*OctArp; // Nombre total de note a transferer
      //randomSeed(k*SequenceArp[0]);
      // lcd.setCursor(14,0);
      //lcd.print(k,DEC);
      //lcd.print(" ");  
      IndexSeq=0;   
      for (i=0; i<k; i++ ) {
          j=random(k);
          SequenceArp[IndexSeq]=SequenceArp[j+24];
          IndexSeq++;  
      }
     
      SequenceArp[IndexSeq]=0;
      memset(SequenceArp+24,0, 24);
      //lcd.setCursor(14,0);
      //lcd.print(k,DEC);
      break;  
    case MOTIF_132:
            switch(NbNoteArp) {
                case 1 :
                   SequenceArp[0]=NoteArp[0];
                   IndexSeq=1;
                   break;
                case 2 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[1];
                   IndexSeq=2;
                   break;
                case 3 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[2];
                   SequenceArp[2]=NoteArp[1];
                   SequenceArp[3]=NoteArp[2];
                   IndexSeq=4;
                   break;
                case 4 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[2];
                   SequenceArp[2]=NoteArp[1];
                   SequenceArp[3]=NoteArp[3];
                   IndexSeq=4;
                   break;
                case 5 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[2];
                   SequenceArp[2]=NoteArp[1];
                   SequenceArp[3]=NoteArp[3];
                   SequenceArp[4]=NoteArp[2];
                   SequenceArp[5]=NoteArp[4];
                   IndexSeq=6;
                   break;
                case 6 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[2];
                   SequenceArp[2]=NoteArp[1];
                   SequenceArp[3]=NoteArp[3];
                   SequenceArp[4]=NoteArp[2];
                   SequenceArp[5]=NoteArp[4];
                   SequenceArp[6]=NoteArp[3];
                   SequenceArp[7]=NoteArp[5];
                   IndexSeq=8;
                   break;
                case 7 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[2];
                   SequenceArp[2]=NoteArp[1];
                   SequenceArp[3]=NoteArp[6];
                   SequenceArp[4]=NoteArp[2];
                   SequenceArp[5]=NoteArp[4];
                   SequenceArp[6]=NoteArp[3];
                   SequenceArp[7]=NoteArp[5];
                   IndexSeq=8;
                   break;
                case 8 :
                   SequenceArp[0]=NoteArp[0];
                   SequenceArp[1]=NoteArp[7];
                   SequenceArp[2]=NoteArp[1];
                   SequenceArp[3]=NoteArp[6];
                   SequenceArp[4]=NoteArp[2];
                   SequenceArp[5]=NoteArp[4];
                   SequenceArp[6]=NoteArp[3];
                   SequenceArp[7]=NoteArp[5];
                   IndexSeq=8;
                   break;
            }
            
            k=IndexSeq;
            for (i=1; i<OctArp; i++ ) {     // Pour chaque octave
              for (j=0; j<k; j++) {  // Pour chaque note appuyee
                  SequenceArp[IndexSeq]=SequenceArp[IndexSeq-k]+12;
                  IndexSeq++; 
              }
            }




    break;          
  }
} 


//---------------------- 
// Handler Note Off Arpegiateur
//---------------------- 
void  MyHandleNoteOffArp(byte channel, byte pitch, byte velocity) {
    CountNote--;              // Decrement du nombre de note
    if (CountNote >128)      // Test si negatif (impossible normalement)
        CountNote=0;          // Si c'est le cas RAZ
    if (CountNote==0)  {       // S'il ne reste plus de note, mettre GATE a zero
        digitalWrite(GATE1,LOW);
		if (LastNoteMidiArp) {
			MIDI.sendNoteOff(LastNoteMidiArp,0,1);
			LastNoteMidiArp=0;
		}
	}
    // cherche la note arretee dans le tableau
    // si toutefois elle est presente
    for (int i=0; i<8; i++) {
        if (NoteArp[i]==pitch) {    // Si note trouvee
           NoteArp[i]=99;           // la remettre a zero
           NbNoteArp--;             // Decremente le nombre de note stockee
           TriNoteArp();
           ReBuildSequenceArp();
        }
     }
   
  lcd.setCursor(1,0);
  lcd.print(NbNoteArp,DEC);
     
}

//--------------------- 
// Handler Note On Arpegiateur
//--------------------- 
void MyHandleNoteOnArp(byte channel, byte pitch, byte velocity) {
int cv2;
   if (!ArpStart)
       return;
  // Si velocity == 0 alors on a recu un note off
  //---------------------------------------------
  if (velocity==0) {
     MyHandleNoteOffArp(channel,pitch,velocity);
    // gestion du note off et sortie du handler
    //MyHandleNoteOffCV(channel, pitch, velocity);
    return;
  }
  CountNote++;    // Incrementation du nombre de note actives
  
  // Si countnote coherent et inferieur a limite maxi
  // alors on a suffisamment de note pour le buffer
  // Stockage de la note dans le tableau des notes
  // et tri du tableau
  if (CountNote>0 and CountNote<MAXNOTEARP+1) {
      NoteArp[CountNote-1]=pitch;   // Stokage de la note jouee
      TriNoteArp();                 // Tri du tableau de note
      NbNoteArp++;                  // Incremente le nombre de note stockee
      ReBuildSequenceArp();
  }
  lcd.setCursor(1,0);
  lcd.print(NbNoteArp,DEC);
  
}



//-----------------------------------------
// Arpegiateur
// Synchro horloge midi
// Entree note clavier midi
// Sortie CV/GATE
// Choix du type de CV (V/Oct, Hz/Volt, sx)
//-----------------------------------------
void Arpegiateur(void) {
byte i;
  GateOff();
  ClockMidi=0;
  ArpStart=0;
  //for (i=0;i<100;i++)
  //    MIDI.read();        
  MIDI.setHandleNoteOn(MyHandleNoteOnArp);      // Definit handler note on 
  MIDI.setHandleNoteOff(MyHandleNoteOffArp);    // Definit handler note off
  MIDI.setHandleClock(MyHandleClockArp);        // Definit handler horloge
  MIDI.setHandleStart(MyHandleStartArp);        // Definit handler  start
  MIDI.setHandleStop(MyHandleStopArp);          // Definit handler  stop
  MIDI.setHandleContinue(MyHandleContinueArp);  // Definit handler  Continue
  InitArp();  
  lcd.clear();                     // RAZ LCD
  lcd.print(F("[x] Gate:000ms"));   // Titre du module active
  lastvalpot=0;
  aff_menu_MidiArp();               // affiche les parametres en cours
  while (1) {                      // boucle infinie
    MIDI.read();                  // lecture du buffer midi et traitement
    if (CountNote>0) {
      if (millis()-StartGate>+valpot) {
        digitalWrite(GATE1,LOW);
		MIDI.sendNoteOff(LastNoteMidiArp,0,1);
		LastNoteMidiArp=0;
        StartGate=millis();
/*
        lcd.setCursor(14,0);
        lcd.print("w");
        
        lcd.setCursor(14,0);
        lcd.print(" ");
        
*/      
      }  
    }
    gestion_menu_MidiArp();        // test touche de fonctions
    if (readButtons(BNAV)==B_ESC) // test si touche ESC enfoncée
      break;                    // Si oui, on sort de la boucle
  }
  MIDI.disconnectCallbackFromType(midi::Continue); // Deconnexion handler continue
  MIDI.disconnectCallbackFromType(midi::Stop); // Deconnexion handler stop
  MIDI.disconnectCallbackFromType(midi::Start); // Deconnexion handler start
  
  MIDI.disconnectCallbackFromType(midi::Clock); // Deconnexion handler clock
  MIDI.disconnectCallbackFromType(midi::NoteOff); // Deconnexion handler note off
  MIDI.disconnectCallbackFromType(midi::NoteOn);  // Deconnexion handler note on
}

//--------------------------------
// Menu pour le module arpegiateur
//--------------------------------
void aff_menu_MidiArp(void) {
    lcd.setCursor(0,1);
    lcd.print(F("x"));
    lcd.print(ListeClockMenu[ClockDivider]);  // Affiche diviseur clock
    lcd.print(F(" "));
    lcd.printByte(CAR_SEP);
    
    lcd.print(OctArp,DEC);                    // Affiche etendue en octave
    lcd.print(F("oc"));
    lcd.printByte(CAR_SEP);
    strcpy_P(BuffAff, (char*)pgm_read_word(&(MotifsArp[NumMotifArp])));
    lcd.print(BuffAff); 
    lcd.printByte(CAR_SEP);
    switch(AModeCV) {
        case VOCT:
            lcd.print(F("Oct"));
          break;
        case HVOLT:
            lcd.print(F("Hz."));
          break;
        case SX150:
            lcd.print(F("SX1"));
          break;
    }
    
}

//--------------------------------
// Gestion des touches arpegiateur
//--------------------------------
void gestion_menu_MidiArp(void) {
    // Lire la valeur du bouton
    valbouton=readButtons(BFUNC); 
    // LIre la valeur du potentiometre
    valpot=readPotArp(POT);
    // Afficher la valeur si elle a changee
    if (lastvalpot!=valpot) {
        lastvalpot=valpot;
        lcd.setCursor(9,0);
        if (valpot<100)
            lcd.print(" ");
        if (valpot<10)
            lcd.print(" ");
        lcd.print(valpot,DEC);    
    }
    
    if (valbouton==0)     // Si aucun bouton appuye on sort
        return;           // pour ne pas perdre de temps
        
    rafLCD=0;
    switch(valbouton) {
        case B_MAINT:
            break;
        case B_F1:              // Division de la clock
            if (ClockDivider==ITEMS_CLOCK) // Si valeur maxi atteinte
                ClockDivider=0;  // Remettre la valeur mini
            else
                ClockDivider++;        // sinon incrementer le diviseur
            digitalWrite(GATE1,LOW);
			if (LastNoteMidiArp) {
				MIDI.sendNoteOff(LastNoteMidiArp,0,1);
				LastNoteMidiArp=0;
			}			
            ClockMidi=0;  
            rafLCD=1;           // Rafraichir LCD
            break;
        case B_F2:              // Nombre octave
            if (OctArp==3)
               OctArp=1;
            else
               OctArp++;
            ReBuildSequenceArp();   
            rafLCD=1;           // Rafraichir LCD
            break;
        case B_F3:              // Sens
              if (NumMotifArp==ITEMS_MOTIFS_ARP-1)
                  NumMotifArp=0;
              else 
                  NumMotifArp++;
            ReBuildSequenceArp();      
            rafLCD=1;           // Rafraichir LCD
            break;
        case B_F4:              // Mode du CV
           if (AModeCV<3)
                AModeCV++;
            else
                AModeCV=1;
            rafLCD=1;
            break;
    }
    if (rafLCD==1) {            // Si flag affichage
        aff_menu_MidiArp();  // Rafraichir menu note
        rafLCD=0;
    }
}
