//----------------------------------------------------
//----------------------------------------------------
//-----------------  MIDI NOTE CV --------------------
//----------------------------------------------------
//----------------------------------------------------

//-----------------------------------------
// HANDLER CC MIDI -> CV
// Ecoute sur le modulation controller (1)
// Procedure pour convertisseur midi -> CV
//----------------------------------------
void MyHandleCtrlCV(byte channel, byte number, byte value) {
  if (ModeCtrl==OFF)      // Sortie si mode CC a off
      return;
  if (number!=1)          // si different de modulation controller on sort
     return;
  vdac=value*32;          // Mapper sur 12bits 
  writeDAC(DAC3,vdac);    // ecrire dans le DAC 3
}


//---------------------------- 
// HANDLER NOTE OFF MIDI -> CV
//---------------------------- 
void  MyHandleNoteOffCV(byte channel, byte pitch, byte velocity) {
    CountNote--;              // Decrement du nombre de note
    if (CountNote ==255)         // Test si negatif (impossible normalement)
        CountNote=0;          // Si c'est le cas RAZ
    if (CountNote==0)         // S'il ne reste plus de note, mettre GATE a zero
        digitalWrite(GATE1,LOW);
}
                                                                                             
//--------------------------- 
// HANDLER NOTE ON MIDI -> CV
//--------------------------- 
void MyHandleNoteOnCV(byte channel, byte pitch, byte velocity) {
int cv2;
  // Si velocity == 0 alors on a recu un note off
  //---------------------------------------------
  if (velocity==0) {
     MyHandleNoteOffCV(channel,pitch,velocity);
    // gestion du note off et sortie du handler
    //MyHandleNoteOffCV(channel, pitch, velocity);
    return;
  }
  
  vdac=0;
  // test du type de CV
  switch(ModeCV) {
      // Volt par octave
      case VOCT :
          if (pitch>=36 and pitch <=96) {
              vdac = pgm_read_word_near( VOct + pitch - 36);
          } 
          else
             return;
          break;
      // Hertz par volt
      case HVOLT :
          if (pitch>=36 and pitch <=87) {
              vdac = pgm_read_word_near( HVolt + pitch - 36);
          } 
          else
             return;
          break;
      // SX150
      case SX150 :
          break;
  };
  CountNote++;    // Incrementation du nombre de note actives

  // Si mode RETRIG et qu'une note etait deja appuyee avant celle en cours
  // il faut redeclencher le gate donc on le passe à 0
  if (ModeMidi==RETRIG and CountNote>1) {
      digitalWrite(GATE1,LOW);
      delay(5);
      
      }
  // Ecriture dans le DAC1 de la hauteur de note si elle an change
  // en cas de repetition, on ne la reecrit pas dans le dac
  if (lastnote!=vdac) {
    writeDAC(DAC1,vdac);
    lastnote=vdac;
  }
  // Si mode velocite actif
  // 
  if (ModeVelo==ON) {
      vdac=velocity*32;
      writeDAC(DAC2,vdac);
  }
  
  // On ouvre le GATE pour jouer la note en cours
  if (ModeMidi==MONO and CountNote==1)
      digitalWrite(GATE1,HIGH);
  if (ModeMidi==RETRIG)
      digitalWrite(GATE1,HIGH);
}



//---------------------------------------------
// Sous-programme conversion midi note vers CV
// Gere le mode Volt/octave, Hertz/Volt, etc...
//---------------------------------------------
void ConversionMidiNoteCV() {
  MIDI.setHandleNoteOn(MyHandleNoteOnCV);       // Definit handler note on 
  MIDI.setHandleNoteOff(MyHandleNoteOffCV);     // Definit handler note off
  MIDI.setHandleControlChange(MyHandleCtrlCV);  // Definit handler CC
  lcd.clear();                     // RAZ LCD
  lcd.print(F("** MIDI -> CV **"));   // Titre du module active
  aff_menu_MidiNoteCV();               // affiche les parametres en cours
  while (1) {                      // boucle infinie
    MIDI.read();                  // lecture du buffer midi et traitement
    gestion_menu_MidiNoteCV();        // test touche de fonctions
    if (readButtons(BNAV)==B_ESC) // test si touche ESC enfoncée
      break;                    // Si oui, on sort de la boucle
  }
  MIDI.disconnectCallbackFromType(midi::ControlChange); // Deconnexion handler CC
  MIDI.disconnectCallbackFromType(midi::NoteOff); // Deconnexion handler note off
  MIDI.disconnectCallbackFromType(midi::NoteOn);  // Deconnexion handler note on
}


//--------------------------------------------
// Menu pour le module conversion midi vers CV
//--------------------------------------------
void aff_menu_MidiNoteCV(void) {
    lcd.setCursor(0,1);
    switch(ModeCV) {
        case VOCT:
            lcd.print(F("Oct"));
          break;
        case HVOLT:
            lcd.print(F("Hz."));
          break;
        case SX150:
            lcd.print(F("SX1"));
          break;
    }
    lcd.printByte(CAR_SEP);
    //lcd.print(" ");
    
    if (ModeMidi==MONO)
        lcd.print(F("MON"));   //monophonique
    else
        lcd.print(F("RTG"));   // ou retrig
        
    lcd.printByte(CAR_SEP);
    //lcd.print(" ");
    if (ModeVelo==ON)
        lcd.print(F("Vel"));   // gestion velocite
    else {
        lcd.print(F("Off"));   // ou pas
        
    }

    lcd.printByte(CAR_SEP);
    //lcd.print(" ");

    if (ModeCtrl==ON)
        lcd.print(F("Ctl"));   // gestion controleur 2
    else
        lcd.print(F("Off"));   // ou pas
}


//----------------------------------------------
// Gestion des touches convertisseur midi -> CV
//----------------------------------------------
void gestion_menu_MidiNoteCV(void) {
    // Lire la valeur du bouton
    valbouton=readButtons(BFUNC);
    if (valbouton==0)     // Si aucun bouton appuye on sort
        return;           // pour ne pas perdre de temps
        
    rafLCD=0;
    switch(valbouton) {
        case B_MAINT:           // Bouton maintenu appuye
            break;
        case B_F1:              // Choix Mode CV
            if (ModeCV<3)
                ModeCV++;
            else
                ModeCV=1;
            rafLCD=1;    
            break;
        case B_F2:            // Choix mode midi
            if (ModeMidi==MONO)
                ModeMidi=RETRIG;
            else
                ModeMidi=MONO;
            CountNote=0;
            digitalWrite(GATE1,LOW);
            rafLCD=1;    
            break;
        case B_F3:            // Choix mode gestion velocite
            if (ModeVelo==ON)
                ModeVelo=OFF;
            else
                ModeVelo=ON;
            CountNote=0;
            digitalWrite(GATE1,LOW);    
            writeDAC(DAC2,0);    
            rafLCD=1;    
            break;
        case B_F4:            // Choix mode controleur
            if (ModeCtrl==ON)
                ModeCtrl=OFF;
            else
                ModeCtrl=ON;
            CountNote=0;
            digitalWrite(GATE1,LOW);    
            writeDAC(DAC3,0);    
            rafLCD=1;    
            break;
    }
    if (rafLCD==1) {
        aff_menu_MidiNoteCV();
        rafLCD=0;
    }
}
