//------------------------------------------------------------------------
// Projet MIDICV
// Module gestion MIDI -> CV/GATE
// Tools analogique : quantizer, sequencer, etc...
//------------------------------------------------------------------------
// Frederic ABIGNOLI 
// v0.8 : 23/01/2013 (initialisation du projet
//------------------------------------------------------------------------
// MOdifications :
// v1.0 : 07/08/2014 : ajout sequencer trigger euclidien
// v1.1 : 14/01/2018 : Midi sur sequenceur euclidien
// v1.2 : 22/01/2021 : Modification code pour arduino >1.8
//                     et nouvelle bibliotheque MIDI (deconnexion handler)
//                     ajout arpegiateur en midi
//                     suppression du midi-thru qui est acti par defaut
//------------------------------------------------------------------------


//------------------------------------------------------
// Bibliotheque pour gerer lecture et ecriture en EEPROM
//------------------------------------------------------
#include <EEPROM.h>

//----------------------------------------------
// Bibliotheque et definitions pour gerer le LCD
//----------------------------------------------
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif
LiquidCrystal_I2C lcd(0x27,16,2);  // sLCD adress 0x27, 16 car, 2 ligne
byte rafLCD;                        // Flag rafraichissement LCD
uint8_t symbup[8]  = {0x4,0xe,0xf5,0x4,0x4,0x4,0x4};    //car fleche up
uint8_t symbdown[8]  = {0x4,0x4,0x4,0x4,0xf5,0xe,0x4};  //car fleche down
uint8_t CARV1[8] = {
  B11111,
  B11111,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
};
uint8_t CARV2[8] = {
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
  B00000,
  B00000,
  B00000,
};
uint8_t CARV3[8] = {
  B11111,
  B11111,
  B00000,
  B11111,
  B11111,
  B00000,
  B00000,
  B00000,
};
uint8_t CARV4[8] = {
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
};
uint8_t CARV5[8] = {
  B11111,
  B11111,
  B00000,
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
};
uint8_t CARV6[8] = {
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
  B00000,
  B11111,
  B11111,
};
uint8_t CARV7[8] = {
  B11111,
  B11111,
  B00000,
  B11111,
  B11111,
  B00000,
  B11111,
  B11111,
};
#define CAR_RIGHT 126   // Numero car fleche vers la droite 
#define CAR_LEFT  127   // Numero car fleche vers la gauche
#define CAR_UP    0     // Numero car fleche vers le haut
#define CAR_DOWN  1     // Numero car fleche vers le bas
#define CAR_SEP   124   // Trait vertical de séparation
 

// --------------------------------
// declaration des pins sur ARDUINO
// --------------------------------
#define MIDIIN  0   // Entree MIDI
#define MIDIOUT 1   // Sortie MIDI
#define CLOCKIN 2   // Pin D2 : clock externe 
#define RESETIN 3   // Pin D3 : reset externe
#define GATE1   4   // Pin D4 : commande GATE 1
#define GATE2   5   // Pin D5 : commande GATE 2 
#define GATE3   6   // Pin D6 : commande GATE 3
#define DAC1    7   // Pin D7 pour DAC no 1
#define DAC2    8   // Pin D8 pour DAC no 2
#define DAC3    9   // Pin D9 pour DAC no 3
// PINS D10 a D13 reservees SPI
//------------------------------------------------------
#define BFUNC   0   // Pin A0 : Boutons Fonctions
#define BNAV    1   // Pin A1 : Boutons Navigations
#define POT     2   // Pin A2 : Potentiometre entree de DATAS
#define CVIN    3   // Pin A3 : Entree analogique CVIN
#define LCDSDA  4   // Pin A4 :  Sortie SDA LCD
#define LCDSCL  5   // Pin A5 :  Sortie SCL LCD
//------------------------------------------------------

// Valeur renvoyee par les boutons de commandes
//---------------------------------------------
#define B_UP    1   // Touche UP
#define B_DOWN  2   // Touche DOWN
#define B_ESC   3   // Touche ESC
#define B_OK    4   // Touche OK
#define B_F1    1   // Touche fonction 1
#define B_F2    2   // Touche fonction 2
#define B_F3    3   // Touche fonction 3
#define B_F4    4   // Touche fonction 4
#define B_MAINT 9   // Derniere touche maintenue appuyee
byte lastbouton[2]; // Stockage du dernier etat dun bouton

// Divers
//-------
#define ON  1
#define OFF 0



//------------------------------------------------
// Bibliotheque et declarations pour gerer le midi
//------------------------------------------------
#include <MIDI.h>   // Midi Library

#define MONO    0   // note on sur note active -> pas de GATE 
#define RETRIG  1   // note on sur note actice -> redeclenchement dun GATE
#define MAXCTRL 31  // n° controleur maxi pour convertisseur ctrl -> CV
#define MINCTRL 20  // n° controleur mini pour convertisseur ctrl -> CV
#define MAXNOTE 60  // Note maxi
#define MINNOTE 36  // Note mini
#define MAXNOTEARP  8
#define TAILLESEQUENCEARP  48 // 3x (3 octave) et 2x (up/down) donc 6 fois la taille des notes de la memoire arpegiateur
#define NOTEBASEEUCLID	36	// Valeur de note pour declenchement sequenceur euclidien
byte CountNote;     // Nombre de note appuyees sur le clavier a un instant "t"
byte LastCountNote; // Derniere valeur de midi note pour voir la difference                      
byte ModeMidi;      // Mode MONO ou RETRIG
byte ModeVelo;      // Interpretation de la velocite
byte ModeCtrl;      // Interpretation du controleur breath (no 2)
int lastnote=0;     // Valeur du dac de la derniere note
byte Ctrl1,Ctrl2,Ctrl3; // Valeur des controleurs pour Ctrl->CV 
byte Note1,Note2,Note3; // Valeur des notes pour 3 notes midi -> 3 gates
byte ClockMidi;     //  Horloge maitre midi en IN
byte ClockDivider;  // Diviseur de l'horloge maitre midi
byte LastClock;     // deniere valeur de clock
byte NoteArp[MAXNOTEARP]; // Tableau des notes de larpegiateur
byte SequenceArp[TAILLESEQUENCEARP]; // Tableau des motifs de larpegiateur
byte NbNoteArp;     // Nombre de note appuyee et stockee dans NoteArp
byte LastNbNoteArp; // Derniere valeur de NbNoteArp
byte NumMotifArp;   // Numero du motif Arpegiateur
byte ArpStart;      // Flag depart arpegiateur
byte OctArp;        // Nombre octave couverture arpegiateur
int PosSeq;        // Position en cours dans la sequence
long StartGate;     // Time du gate on
long StartGateEuc[3];     // Time du gate on pour les 3 patterns sequenceur euclidien
byte GateEucState[3];		// Etat des 3 gates du sequenceur euclidien
byte LastNoteMidiArp;	// Deniere note jouee dans arpegiateur
byte QNote;         // type note quantizer (blanches ou toutes les notes)
#define BLANCHES  0 // Notes blanches
#define SEMITONES 1 // Toutes les notes
int ceuclid[3];		// Sequence euclidienne 'n'
byte beuclid[3];	// nombre de beat de la sequence 'n'
byte teuclid[3];	// nombre total de pas de la sequence 'n'
byte xm_euc,ym_euc;	// coordonnee menus sequencer euclidien
int PosSeqEuclidian[3];
byte SeqEuclideanStart;      // Flag depart sequenceur euclidien


byte ListeClockMenu[]={
    1,2,3,4,6,8
};
#define ITEMS_CLOCK 5

MIDI_CREATE_DEFAULT_INSTANCE();

//------------------------------------------------
// Bibliotheque et declarations pour gerer les DAC
//------------------------------------------------
#include "SPI.h"      // SPI library
int del=0;            // used for various delays
word outputValue = 0; // a word is a 16-bit number
byte data = 0;        // and a byte is an 8-bit number
int vdac;             // valeur a ecrire dans les DAC
int lastvdac;         // Stockage ancienne valeur vdac

// Tableaux de conversion midi -> CV
//----------------------------------
#include "valeursmem.h"    // Valeur DAC suivant note midi

// ---------------------------- 
// declarations gestion CV/GATE
// ---------------------------- 
#define VOCT    1         // Mode Volt par octave
#define HVOLT   2         // Mode Hertz par volt
#define SX150   3         // Mode SX150
byte  ModeCV;             // Mode CV actif (Octave, Hertz, SX150...)
byte  QModeCV;            // Mode CV actif (Oct, Hertz, SX150) pour quantizer
byte  SModeCV;            // Mode CV actif (Oct, Hertz, SX150) pour sequenceur
byte  AModeCV;            // Mode CV actif (Oct, Hertz, SX150) pour arpegiateur 
#define DELAY_RETRIG  5   // Delai du retrig en ms

// Declarations pour arpegiateur
char BuffAff[18];     // Buffer pour transferer donnees en FLASH
byte PosMenu;         // Position courante dans un menu
int valbouton;        // Valeur du bouton appuye
int valpot;           // Valeur du potentiometre
int lastvalpot;       // Ancienne valeur du potentiometre
int i,j;              // Variables pour les boucles
//------------------------------------------------------------------
// Initialisation de l'EEPROM
// avec des valeurs en dur, si demarrage avec touche ESC
//------------------------------------------------------------------
void EEPROM_Init(void) {
    EEPROM.write(0, 1);		// Mode midi on
    EEPROM.write(1, 1);		// Mode Velo on
    EEPROM.write(2, 1);		// Mode Ctrl on
    EEPROM.write(3, 1);		// Mode CV convertisseur (V/Oct)
    EEPROM.write(4, 24);	// Numero controller 1
    EEPROM.write(5, 25);	// Numero controller 2
    EEPROM.write(6, 26);	// Numero controller 3
    EEPROM.write(7, 1);		// Mode CV quantizer (V/Oct)
    EEPROM.write(8, 0);		// Mode CV sequenceur (V/Oct)
    EEPROM.write(9, 36);	// Note 1 declencheur gate
    EEPROM.write(10, 38);	// Note 2 declencheur gate
    EEPROM.write(11, 40);	// Note 3 declencheur gate
    EEPROM.write(12, 3);	// Clock divider (x4)
    EEPROM.write(13, 1);	// Mode CV Arpegiateur (V/Oct)
    EEPROM.write(14, 6);	// Motif arpegiateur 
    EEPROM.write(15, 3);	// Nbre Oct arpegiateur
    EEPROM.write(16, 1);	// Quantizer note semi-tone
    EEPROM.write(17, 5);	// nbre beat euclid trk 1
    EEPROM.write(18, 5);	// nbre beat euclid trk 2
    EEPROM.write(19, 6);	// nbre beat euclid trk 3
    EEPROM.write(20, 16);	// nbre steps euclid trk 1
    EEPROM.write(21, 14);	// nbre steps euclid trk 2
    EEPROM.write(22, 16);	// nbre steps euclid trk 3
}


//------------------------------------------------------------------
// Sauvegarde des parametres du boitier dans l'EEPROM
// Les donnees seront relues automatiquement au demarrage du boitier
//------------------------------------------------------------------
void EEPROM_Save(void) {
    EEPROM.write(0, ModeMidi);
    EEPROM.write(1, ModeVelo);
    EEPROM.write(2, ModeCtrl);
    EEPROM.write(3, ModeCV);
    EEPROM.write(4, Ctrl1);
    EEPROM.write(5, Ctrl2);
    EEPROM.write(6, Ctrl3);
    EEPROM.write(7, QModeCV);
    EEPROM.write(8, SModeCV);
    EEPROM.write(9, Note1);
    EEPROM.write(10, Note2);
    EEPROM.write(11, Note3);
    EEPROM.write(12, ClockDivider);
    EEPROM.write(13, AModeCV);
    EEPROM.write(14, NumMotifArp);
    EEPROM.write(15, OctArp);
    EEPROM.write(16, QNote);
    EEPROM.write(17, beuclid[0]);
    EEPROM.write(18, beuclid[1]);
    EEPROM.write(19, beuclid[2]);
    EEPROM.write(20, teuclid[0]);
    EEPROM.write(21, teuclid[1]);
    EEPROM.write(22, teuclid[2]);
}

//---------------------------------------------
// Lecture des parametres sauvegardes en EEPROM
//---------------------------------------------
void EEPROM_Read(void) {
    ModeMidi=EEPROM.read(0);
    ModeVelo=EEPROM.read(1);
    ModeCtrl=EEPROM.read(2);
    ModeCV=EEPROM.read(3);
    Ctrl1=EEPROM.read(4);
    Ctrl2=EEPROM.read(5);
    Ctrl3=EEPROM.read(6);
    QModeCV=EEPROM.read(7);
    SModeCV=EEPROM.read(8);
    Note1=EEPROM.read(9);
    Note2=EEPROM.read(10);
    Note3=EEPROM.read(11);
    ClockDivider=EEPROM.read(12);
    AModeCV=EEPROM.read(13);
    NumMotifArp=EEPROM.read(14);
    OctArp=EEPROM.read(15);
    QNote=EEPROM.read(16);
    beuclid[0]=EEPROM.read(17);
    beuclid[1]=EEPROM.read(18);
    beuclid[2]=EEPROM.read(19);
    teuclid[0]=EEPROM.read(20);
    teuclid[1]=EEPROM.read(21);
    teuclid[2]=EEPROM.read(22);
}

//--------------------------
// Remise a zero des 3 gates
//--------------------------
void GateOff(void) {
  digitalWrite(GATE1,LOW);
  digitalWrite(GATE2,LOW); 
  digitalWrite(GATE3,LOW);
}

//---------------------------------
// Lit la valeurs du potentiomètre
// Renvoie une valeur qui determine
// la longueur du GATE en ms
//---------------------------------
int readPotArp(int pin) {
int c=0;
  // Lecture de la valeur du potentiometre
  // Renvoie une valeur de 0 a 1023
  // En fonction de la valeur du ClockDivider
  // On map a la sortie des valeurs differentes
  c=analogRead(pin);
  switch(ListeClockMenu[ClockDivider]) {
      case 1:
        c = map(c, 0, 1023, 2, 500);    //Clock x1 -> 500ms max
        break;
      case 2:
        c = map(c, 0, 1023, 2, 350);    //Clock x2 -> 350ms max
        break;
      case 3:
        c = map(c, 0, 1023, 2, 250);    //Clock x3 -> 250ms max
        break;
      case 4:
        c = map(c, 0, 1023, 2, 200);    //Clock x4 -> 200ms max
        break;
      case 6:
        c = map(c, 0, 1023, 2, 150);    //Clock x6 -> 150ms max
        break;
      case 8:
        c = map(c, 0, 1023, 2, 100);    //Clock x8 -> 100ms max
        break;
      // Si clockdivider indefini ou sur une valeur non prevue
      default:
        c = map(c, 0, 1023, 2, 150);    //Clock x6 -> 500ms max
        break;
  }   
  return c;
}

//-------------------------------------------
// Lit les boutons et renvoie la valeurs lues
// Parametre BFUNC ou BNAV
// Ne permet qu'une saisie quand un bouton
// est appuyé
//-------------------------------------------
int readButtons(int pin) {
int b,c = 0;
  c=analogRead(pin);      // Lit lentree analogique 
    if (c>1000) {
      lastbouton[pin]=0;  // Remettre flag etat a zero
      return 0;           // Pas de bouton appuyé sortir
  }
  // Arrive ici normalement un bouton a ete appuye
  for (int i=1; i<20; i++) {
    MIDI.read();
    delay(1);              // Attente
  }
  
  c=analogRead(pin);      // Relire pour debouncing 
  if (c>1000) {
      lastbouton[pin]=0;  // Remettre flag etat a zero
      return 0;           // Toujours pas de bouton appuye
  }
 
  if (lastbouton[pin]>0)   // Si le dernier bouton n'a pas ete relache
      return 9;   // renvoit code+10 (touche 1 = 11, 2=12, etc)
  // Arrivé ici on est sur que le bouton a ete appuye
  if (c>200 && c<300) {
      lastbouton[pin]=1;
      return 1;           // bouton 1 appuye
  } 
  if (c>450 && c<550) {
      lastbouton[pin]=2;
      return 2;           // bouton 2 appuye
  } 
  if (c>750 && c<850) {
      lastbouton[pin]=3;
      return 3;           // bouton 3 appuye
  }                       
  if (c>950 && c<999) {
        lastbouton[pin]=4;
      return 4;           // bouton 4 appuye
  }
}


//------------------------------
// Fonction ecriture dans un DAC
// Param 1 : Numero du dac
// Param 2 : donnee
//------------------------------
void writeDAC(uint8_t numdac, uint16_t datain) {
    
    outputValue = datain;           // recuperation de la donnee
    digitalWrite(numdac, LOW);      // Mettre a l'ecoute le bon dac
    data = highByte(outputValue);   // Extraire la partie MSB des donnees
    data = 0b00001111 & data;       // Nutiliser que les 4 bits de poids faibles
    data = 0b00110000 | data;       // Mettre les bits 4 et 5 à 1
    SPI.transfer(data);             // transferer la permiere partie
    data = lowByte(outputValue);    // extraire la partie LSB des donnees
    SPI.transfer(data);             // transferer les donnees
    digitalWrite(numdac, HIGH);     // rentre le DAC disable en entree
}



//------------------------------
// Generation de fonction random
// Libre ou synchronisee
//------------------------------
void RandomWave(void) {

}


//--------------------------------
// Creation de son avec les 3 DAC
// Declenchement avec 3 notes midi
//--------------------------------
void RhythmMachine(void) {
}




//---------------------------------------
// Menu principal du boitier
// Affiche les fonctionnalites du boitier
//---------------------------------------
void aff_menu_principal() {
  GateOff();
  PosMenu=1;
  lcd.clear();
  
  strcpy_P(BuffAff, (char*)pgm_read_word(&(MenuPrincipal[1])));
  lcd.print(BuffAff); 
//  lcd.print(MenuPrincipal[1]);

  // lcd.setCursor(0,1);
  // lcd.print(freeMemory(),DEC);
  
  lcd.setCursor(6,1);
  lcd.print("(");
  lcd.printByte(CAR_UP);
  lcd.print("/");
  lcd.printByte(CAR_DOWN);
  lcd.print(F(") [OK]"));
}

//-----------------------------------
// Gestion des touches menu principal
//-----------------------------------
void gestion_menu_principal(void) {

    // Lire la valeur du bouton
    valbouton=readButtons(BNAV);
    switch(valbouton) {
        case B_MAINT:           // Bouton maintenu appuye
            break;
        case B_UP:              // Bouton UP
            if (PosMenu>0) {
                PosMenu--;
                lcd.setCursor(0,0);
                strcpy_P(BuffAff, (char*)pgm_read_word(&(MenuPrincipal[PosMenu])));
                lcd.print(BuffAff); 
                
                //lcd.print(MenuPrincipal[PosMenu]);
            }
            break;
        case B_DOWN:            // Bouton down
            if (PosMenu<ITEMS_MENU_PRINCIPAL-1) {
                PosMenu++;
                lcd.setCursor(0,0);
                strcpy_P(BuffAff, (char*)pgm_read_word(&(MenuPrincipal[PosMenu])));
                lcd.print(BuffAff); 
                // lcd.print(MenuPrincipal[PosMenu]);
            }
            break;
        case B_OK:            // Bouton OK
            switch(PosMenu) {
                case 0:
                    lcd.clear();
                    lcd.print(F("Writing..."));
                    EEPROM_Save();
                    delay(500);
                    break;
                case 1:
                    ConversionMidiNoteCV();
                    break;
                case 2:
                    ConversionMidiCtrlCV();
                    break;
                case 3:
                    ConversionMidiNoteGate();
                    break;
                case 4:
                    Arpegiateur();
                    break;
                case 5:
                    Quantizer();
                    break;
                case 6:
                    RandomWave();
                    break;
                case 7:
                    EuclideanSequencer();              
                    break;
                case 8:
                    RhythmMachine();
                    break;
            }
            aff_menu_principal();
            break;
    }
}


//--------------------
// SETUP DU PROGRAMME
//--------------------
void setup()
{
  // Initialisation du LCD
  //----------------------
  lcd.init();                     // init du lcd 
  lcd.backlight();                // Retroeclairage
  lcd.createChar(0, symbup);      // Creation fleche UP 
  lcd.createChar(1, symbdown);    // Creation fleche DOWN
  lcd.home();                     // curseur vers home
  lcd.print(F("** MIDI TOOLS **"));   // Affichage titre
  lcd.setCursor(0,1);
  lcd.print(F("   Version 1.2  "));   // Affichage titre
  delay(2000);          
  lcd.clear();                    // Efface ecran
  lcd.home();                     // curseur vers home
  rafLCD=0;                       // flag affichage LCD
  

  // Initialisation du MIDI
  //-----------------------
  MIDI.begin(MIDI_CHANNEL_OMNI);              // demarre le midi en mode OMNI
  MIDI.turnThruOff();						  // Empeche recopie midi IN vers midi OUT
											  // utile pour les modules qui genere du midi OUT
											  // avec une entree simultanee (comme arpegiateur)
  CountNote=0;                                // Compteur de note actives
  
  // Initialisation du DAC
  //---------------------- 
  pinMode(10, OUTPUT);
  pinMode(DAC1,OUTPUT);
  pinMode(DAC2,OUTPUT);
  pinMode(DAC3,OUTPUT);
  SPI.begin();                  // Initialisation SPI
  SPI.setBitOrder(MSBFIRST);    // Octet de poid fort en premier
  writeDAC(DAC1,0);             // RAZ DAC1
  writeDAC(DAC2,0);             // RAZ DAC2
  writeDAC(DAC3,0);             // RAZ DAC3
  
  //  Initialisation des GATES
  GateOff();
  
  // Initialisation des menus
  PosMenu=0;
  
  // Initialisation des boutons
  lastbouton[BFUNC]=0;
  lastbouton[BNAV]=0;
  lastvalpot=0;
  // Initialisation de l'EEPROM
  // si touche ESC appuyee au demarrage du midiCV
  valbouton=readButtons(BNAV);
  if (valbouton==B_ESC) {
	lcd.clear();
	lcd.print("Init EEPROM...");
	EEPROM_Init();
	delay(1000);
	lcd.print("OK");
	delay(1000);
  }
  
  EEPROM_Read(); // Lecture des parametre dans lEEPROM


  // affiche menu principal
  aff_menu_principal();  
}


//--------------------
// PROGRAMME PRINCIPAL
//--------------------
void loop() {
// int i;

    gestion_menu_principal();
	/*
	lcd.clear();
	ceuclid[0]=euclid(5,13);
	//ceuclid[0]=4095;
	aff_euclidean( ceuclid[0],13 );
	lcd.setCursor(0,1);
	for (i=0; i<16; i++) {
		if (step_euclidean(ceuclid[0],i))
			lcd.print("X");
		else 
			lcd.print(".");
	}
	
	while(1) {
	}
	*/
}
